/*
 * This application demonstrates how to use the functions provided by the HiResTimer utility
 * Note:
 *		The MCF5272 processor is not compatible with the HiResTimer utility.
 */

#include "predef.h"
#include <stdio.h>
#include <ctype.h>

#ifdef MOD5213
#include <basictypes.h>
#include <serialirq.h>
#include <system.h>
#include <constants.h>
#include <ucos.h>
#include <SerialUpdate.h>
#include <smarttrap.h>
#include <gdbstub.h>

#elif defined MCF52234
#include <basictypes.h>         // Include for variable types
#include <serialpoll.h>         // Use UART interrupts instead of polling
#include <smarttrap.h>          // NetBurner Smart Trap ability
#include <constants.h>          // Include for constants like MAIN_PRIO
#include <system.h>             // Include for system functions
#include <ethernet_diag.h>
#include <netif.h>
#include <ip.h>
#include <taskmon.h>
#include <autoupdate.h>
#include <tcp.h>

#else
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#endif

#if( ( defined MOD5270 ) || ( defined MOD5282 ) || (defined MOD5234) || (defined MOD5213) || (defined MOD5441X)  )
#include <pins.h>
#endif

#include <utils.h>
#include <stdlib.h>
#include <HiResTimer.h>
#include <taskmon.h>

extern "C"
{
void UserMain( void * pd );
}

const char * AppName = "Timer_Example";

DWORD LEDValue = 0;
double TotalTime = 0;
char Buffer[1000];

void IncrementLEDs( );

void SetGPIOtoDMATimerOut();  // Timer pin set to timer out function for driver callback

/**
 * Wait for DHCP address assignment - this function is optional.  In some
 * instances, you may want to wait for a DHCP address before continuing in your
 * program.
 */
#ifdef MCF52234
void WaitForDhcpAddressAssignment( void )
{
   // Wait about 2 seconds for Ethernet link to be established
   WORD ncounts = 0;
   while ( !bEtherLink && ( ncounts < 2 * TICKS_PER_SECOND ) )
   {
      ncounts++;
      OSTimeDly( 1 );
   }

   if ( MyIpAddr == 0 )
   {
      iprintf( "Waiting for DHCP address assignment... \r\n" );

      // Wait about 6 seconds for DHCP address assignment
      ncounts = 0;
      while ( ( MyIpAddr == 0 ) && ( ncounts < 6 * TICKS_PER_SECOND ) )
      {
         ncounts++;
         OSTimeDly( 1 );
      }

      if ( MyIpAddr == 0 ) iprintf( "Failed\r\n" );
   }
}
#endif

void UserMain( void * pd )
{

#ifdef MOD5213
   SimpleUart(0,SystemBaud);
   assign_stdio(0);
   EnableSerialUpdate();
#elif defined MCF52234
   SimpleUart( 0, SystemBaud );   // Initialize UART 0
   assign_stdio( 0 );   // Use UART 0 for stdio

   EnableSmartTraps();    // Enable SmartTrap utility
   EnableTaskMonitor();   // Enable TaskScan utility
   InitializeStack();     // Initialize network stack

   WaitForDhcpAddressAssignment();

   EnableAutoUpdate();                 // Enable network firmware updates
   OSChangePrio( MAIN_PRIO );          // Set standard main task priority (50)

#else
   InitializeStack();
   if( EthernetIP == 0 )
      GetDHCPAddress();
   EnableAutoUpdate();
   EnableTaskMonitor();
#endif

   OSChangePrio( MAIN_PRIO );

   iprintf( "%s application started\n", AppName );

   HiResTimer *timer;


   while( 1 )
   {
      iprintf( "\r\n1) Use DMA Timer to time an operation\r\n" );
      iprintf( "2) Use DMA Timer to repeatedly trigger an interrupt routine\r\n" );
      iprintf( "3) Use DMA Timer to generate a precise OSTimeDly()\r\n" );
      iprintf( "4) Use DMA Timer to generate a precise polling delay\r\n" );
#if ( ( defined MOD5270 ) || ( defined MOD5282 ) || ( defined MOD5234 ) || ( defined MOD5213 ) )
      iprintf( "5) Use DMA Timer to generate a Clock output\r\n" );
#endif
      iprintf( "Select the operation to test :" );
      gets( Buffer );
      iprintf( "\r\n" );
      switch( Buffer[0] )
      {
		 // Using the timer as a stopwatch
         case '1':
         {
        	// Obtain a pointer to timer 0
        	timer = HiResTimer::getHiResTimer(0);
        	// Initialize the timer
        	timer->init();
            iprintf( "Press enter to start HiResTimer... \r\n" );
            gets( Buffer );
            // Start the timer
            timer->start();
            iprintf( "Press enter to stop HiResTimer... \r\n" );
            gets( Buffer );
            // Read the time elapsed
            TotalTime = timer->readTime();
            printf( "The total time elapsed was %g seconds\r\n", TotalTime );
            // Stop and clear the timer
            //iprintf(timer->toString());
            timer->stopClear();
            break;
         }
         // Using the timer to control a periodic event
         case '2':
         {
        	timer = HiResTimer::getHiResTimer(0);
        	// Set the function to be called when
        	timer->setInterruptFunction(IncrementLEDs); // Point timer interrupt call-back to LED function
            iprintf( "Input number of seconds (floating point) between LED increments:" );
            gets( Buffer );
            timer->init(strtod( Buffer, NULL ) );
            timer->start();
            iprintf( "\r\nPress enter to disable incrementing LEDs\r\n" );
            gets( Buffer );
            timer->stopClear();
            timer->clearInterruptFunction();
            LEDValue = 0;
            putleds( 0 );
            break;
         }
         // Demonstrating delay function
         case '3':
         {
            iprintf( "Input number of seconds (floating point) for precise OSTimeDly():" );
            gets( Buffer );
            DWORD StartingTick = TimeTick;
            timer = HiResTimer::getHiResTimer();
            timer->delay(strtod( Buffer, NULL ));
            iprintf( "\r\nDelay finished in %d system ticks\r\n", ( TimeTick - StartingTick ) );
            break;
         }
         // Demonstrating the polling delay function
         case '4':
         {
            iprintf( "Input number of seconds (floating point) for precise polling time delay:" );
            gets( Buffer );
            double DelayTime = strtod( Buffer, NULL );
            DWORD StartingTick = TimeTick;
            timer = HiResTimer::getHiResTimer();
            timer->pollingDelay( DelayTime );
            iprintf( "\r\nDelay finished in %d system ticks\r\n", ( TimeTick - StartingTick ) );
            break;
         }
#if ( ( defined MOD5270 ) || ( defined MOD5282 ) || ( defined MOD5234 ) || ( defined MOD5213 ) )
         case '5':
         {
            //--------------------------------------------------
            // NOTE - You must manually set the correct timer output pin for the
            //        proper timer output function for the clock generator to work
            //--------------------------------------------------
#if( ( defined MOD5270 ) || ( defined MOD5282 ) )
            J2[34].function( PINJ2_34_DTOUT1 );
#elif(defined MOD5234)
            J2[34].function( PINJ2_34_DT1OUT );
#elif(defined MOD5213)
            Pins[23].function( PIN23_DTOUT1 );
#endif

			iprintf( "Input the Frequency (floating point) to generate a clock on timer 1 out:" );
            gets( Buffer );
            timer = HiResTimer::getHiResTimer(1);
            timer->clockGenerator( strtod( Buffer, NULL) );
            iprintf( "\r\n" );
            break;
         }
#endif

         default:
            break;
      }
   }
}

// A simple funtion that increments the LEDs displayed on the dev-board
void IncrementLEDs()
{
   // The Timer parameter is not used here since we do not need to determine which timer interrupted
   LEDValue++;
   iprintf("LEDValue: %d", LEDValue);
#ifndef MCF52234
   putleds( LEDValue );
#endif
}



